/** 
 *   Camera Control for STAR I.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#ifndef _CAMERA_H
#define _CAMERA_H

#define CC_GPIB 0
#define CC_PADDR 1

#define CCD_Y_LEN 385
#define CCD_X_LEN 576

/* camera_init_gpig */

int set_gpib_controller(int gpad);
int wait_camera_controller_on(int pad);
int open_camera_controller(int pad);


/* camera_io_gpib */

#define CMD_OK    0x60
#define CMD_ERROR 0x40
#define RW_BUF_SIZE 80

int my_ibwrite_generic(
	int (*write_func)(int ud, const void *data, long num_bytes),
	int dev, const char *fmt, ...);
int get_int_from_cc(int dev, char *cmd, int *val);
int get_str_from_cc(int dev, char *cmd, char *str, int len);
int put_int_to_cc(int dev, char *cmd, int val);
int put_str_to_cc(int dev, char *cmd, char *str);
int action_to_cc(int dev, char *cmd);

/* camera_cmd */

#include "camera_header.h"

#define GAIN_HIGH    1
#define GAIN_LOW     0
#define TEMP_REG_ON  1
#define TEMP_REG_OFF 0
#define ZOOM_1X      0
#define ZOOM_2X      1
#define ZOOM_3X      2
#define AUTO_WIN_ON  1
#define AUTO_WIN_OFF 0

int get_exposure_time(int dev, int *time);
/*
.
.
.
*/


#endif /* ifndef _CAMERA_H */
