/** 
 *   Camera Control for STAR I.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdarg.h>
#include <unistd.h>
#include <malloc.h>
#include <gpib/ib.h>
#include "camera.h"
#include "my_stdio.h"


int my_ibwrite_generic(
	int (*write_func)(int ud, const void *data, long num_bytes),
	int dev, const char *fmt, ...)
{
	va_list ap;
	int l = -1;
	int n, size = 80;
	char *buff = NULL;

	if ((buff = malloc (size)) == NULL) return l;
	while (1) {
		/* Try to print in the allocated space. */
		va_start(ap, fmt);
		n = vsnprintf (buff, size, fmt, ap);
		va_end(ap);
		if (n > -1 && n < size) {
			break;
		}
		else {
			/* Else try again with more space. */
			if (n > -1)    /* glibc 2.1 */
				size = n+1; /* precisely what is needed */
			else           /* glibc 2.0 */
				size *= 2;  /* twice the old size */
			if ((buff = realloc (buff, size)) == NULL)
				return l;
		}
	}
	if (buff != NULL) {
		PVERB(PINFO,"# Send: %s\n", buff);
#ifdef DEMO_MODE
		l = write(dev, buff, n);
		write(dev, "\n", 1);

#else
		l = write_func(dev, buff, n);
#endif

#ifdef VERBOSE_MODE
		if (l & ERR && !(l & TIMO)) {
			SYSLOG(LOG_ERR,"Error sending: %s [ibstat: 0x%x]\n", 
			      buff, l);
		}
#endif
		free(buff);
	}

	return l;
}


int get_int_from_cc(int dev, char *cmd, int *val)
{
	char ctmp[RW_BUF_SIZE];
	u_char spr;

	if ( my_ibwrite_generic(ibwrt, dev, "@%c", *cmd) & ERR) 
		return -1;
	while (!(ibrsp(dev, &spr) & ERR)) {
		if (spr == CMD_OK) {
			break;
		}else if (spr == CMD_ERROR) {
			return -1;
		}
	}
	if (ibrd(dev, ctmp, RW_BUF_SIZE) & ERR) {
		return -1;
	}else{
		*val = atoi(ctmp);

	}		

	return 0;
}


int get_str_from_cc(int dev, char *cmd, char *str, int len)
{
	u_char spr;

	if ( my_ibwrite_generic(ibwrt, dev, "@%c", *cmd) & ERR) 
		return -1;
	while (!(ibrsp(dev, &spr) & ERR)) {
		if (spr == CMD_OK) {
			break;
		}else if (spr == CMD_ERROR) {
			return -1;
		}
	}
	if (ibrd(dev, str, len) & ERR) {
		str = NULL;
		return -1;
	}

	return 0;
}


int put_int_to_cc(int dev, char *cmd, int val)
{
	u_char spr;

	if ( my_ibwrite_generic(ibwrt, dev, "!%c%d", *cmd, val) & ERR) 
		return -1;
	while (!(ibrsp(dev, &spr) & ERR)) {
		if (spr == CMD_OK) {
			break;
		}else if (spr == CMD_ERROR) {
			return -1;
		}
	}

	return 0;
}


int put_str_to_cc(int dev, char *cmd, char *str)
{
	u_char spr;

	if ( my_ibwrite_generic(ibwrt, dev, "!%c%s", *cmd, str) & ERR) 
		return -1;
	while (!(ibrsp(dev, &spr) & ERR)) {
		if (spr == CMD_OK) {
			break;
		}else if (spr == CMD_ERROR) {
			return -1;
		}
	}

	return 0;
}


int action_to_cc(int dev, char *cmd)
{
	u_char spr;

	if ( my_ibwrite_generic(ibwrt, dev, ":%s", cmd) & ERR) 
		return -1;
	while (!(ibrsp(dev, &spr) & ERR)) {
		if (spr == CMD_OK) {
			break;
		}else if (spr == CMD_ERROR) {
			return -1;
		}
	}

	return 0;
}
